@@ -3,26 +3,26 @@ import numpy as np
 import os.path
 from functools import partial
 from h5py import File, Group, Dataset, special_dtype, SoftLink, ExternalLink, Reference, RegionReference, check_dtype
-from six import raise_from, text_type, string_types, binary_type
 import warnings
-from ...container import Container
 
-from ...utils import docval, getargs, popargs, call_docval_func
-from ...data_utils import AbstractDataChunkIterator, get_shape
+from ...container import Container
+from ...utils import docval, getargs, popargs, call_docval_func, get_data_shape
+from ...data_utils import AbstractDataChunkIterator
 from ...build import Builder, GroupBuilder, DatasetBuilder, LinkBuilder, BuildManager,\
                      RegionBuilder, ReferenceBuilder, TypeMap, ObjectMapper
 from ...spec import RefSpec, DtypeSpec, NamespaceCatalog, GroupSpec
 from ...spec import NamespaceBuilder
 
-from .h5_utils import H5ReferenceDataset, H5RegionDataset, H5TableDataset,\
+from .h5_utils import BuilderH5ReferenceDataset, BuilderH5RegionDataset, BuilderH5TableDataset,\
                       H5DataIO, H5SpecReader, H5SpecWriter
 
 from ..io import HDMFIO, UnsupportedOperation
+from ..warnings import BrokenLinkWarning
 
 ROOT_NAME = 'root'
 SPEC_LOC_ATTR = '.specloc'
-H5_TEXT = special_dtype(vlen=text_type)
-H5_BINARY = special_dtype(vlen=binary_type)
+H5_TEXT = special_dtype(vlen=str)
+H5_BINARY = special_dtype(vlen=bytes)
 H5_REF = special_dtype(ref=Reference)
 H5_REGREF = special_dtype(ref=RegionReference)
 
@@ -30,7 +30,8 @@ H5_REGREF = special_dtype(ref=RegionReference)
 class HDF5IO(HDMFIO):
 
     @docval({'name': 'path', 'type': str, 'doc': 'the path to the HDF5 file'},
-            {'name': 'manager', 'type': BuildManager, 'doc': 'the BuildManager to use for I/O', 'default': None},
+            {'name': 'manager', 'type': (TypeMap, BuildManager),
+             'doc': 'the BuildManager or a TypeMap to construct a BuildManager to use for I/O', 'default': None},
             {'name': 'mode', 'type': str,
              'doc': 'the mode to open the HDF5 file with, one of ("w", "r", "r+", "a", "w-", "x")'},
             {'name': 'comm', 'type': 'Intracomm',
@@ -58,14 +59,17 @@ class HDF5IO(HDMFIO):
 
         if manager is None:
             manager = BuildManager(TypeMap(NamespaceCatalog()))
+        elif isinstance(manager, TypeMap):
+            manager = BuildManager(manager)
         self.__comm = comm
         self.__mode = mode
         self.__path = path
         self.__file = file_obj
-        super(HDF5IO, self).__init__(manager, source=path)
-        self.__built = dict()       # keep track of which files have been read
-        self.__read = dict()        # keep track of each builder for each dataset/group/link
+        super().__init__(manager, source=path)
+        self.__built = dict()       # keep track of each builder for each dataset/group/link for each file
+        self.__read = dict()        # keep track of which files have been read. Key is the filename value is the builder
         self.__ref_queue = deque()  # a queue of the references that need to be added
+        self.__dci_queue = deque()  # a queue of DataChunkIterators that need to be exhausted
         ObjectMapper.no_convert(Dataset)
 
     @property
@@ -101,15 +105,59 @@ class HDF5IO(HDMFIO):
             if namespaces is None:
                 namespaces = list(spec_group.keys())
 
+            readers = dict()
+            deps = dict()
             for ns in namespaces:
                 ns_group = spec_group[ns]
                 latest_version = list(ns_group.keys())[-1]
                 ns_group = ns_group[latest_version]
                 reader = H5SpecReader(ns_group)
+                readers[ns] = reader
+                for spec_ns in reader.read_namespace('namespace'):
+                    deps[ns] = list()
+                    for s in spec_ns['schema']:
+                        dep = s.get('namespace')
+                        if dep is not None:
+                            deps[ns].append(dep)
+
+            order = cls._order_deps(deps)
+            for ns in order:
+                reader = readers[ns]
                 d.update(namespace_catalog.load_namespaces('namespace', reader=reader))
 
             return d
 
+    @classmethod
+    def _order_deps(cls, deps):
+        """
+        Order namespaces according to dependency for loading into a NamespaceCatalog
+
+        Args:
+            deps (dict): a dictionary that maps a namespace name to a list of name of
+                         the namespaces on which the the namespace is directly dependent
+                         Example: {'a': ['b', 'c'], 'b': ['d'], c: ['d'], 'd': []}
+                         Expected output: ['d', 'b', 'c', 'a']
+        """
+        order = list()
+        keys = list(deps.keys())
+        deps = dict(deps)
+        for k in keys:
+            if k in deps:
+                cls.__order_deps_aux(order, deps, k)
+        return order
+
+    @classmethod
+    def __order_deps_aux(cls, order, deps, key):
+        """
+        A recursive helper function for _order_deps
+        """
+        if key not in deps:
+            return
+        subdeps = deps.pop(key)
+        for subk in subdeps:
+            cls.__order_deps_aux(order, deps, subk)
+        order.append(key)
+
     @classmethod
     def __convert_namespace(cls, ns_catalog, namespace):
         ns = ns_catalog.get_namespace(namespace)
@@ -208,7 +256,9 @@ class HDF5IO(HDMFIO):
     @docval({'name': 'container', 'type': Container, 'doc': 'the Container object to write'},
             {'name': 'cache_spec', 'type': bool, 'doc': 'cache specification to file', 'default': True},
             {'name': 'link_data', 'type': bool,
-             'doc': 'If not specified otherwise link (True) or copy (False) HDF5 Datasets', 'default': True})
+             'doc': 'If not specified otherwise link (True) or copy (False) HDF5 Datasets', 'default': True},
+            {'name': 'exhaust_dci', 'type': bool,
+             'doc': 'exhaust DataChunkIterators one at a time. If False, exhaust them concurrently', 'default': True})
     def write(self, **kwargs):
         if self.__mode == 'r':
             raise UnsupportedOperation(("Cannot write to file %s in mode '%s'. "
@@ -216,7 +266,7 @@ class HDF5IO(HDMFIO):
                                        % (self.__path, self.__mode))
 
         cache_spec = popargs('cache_spec', kwargs)
-        call_docval_func(super(HDF5IO, self).write, kwargs)
+        call_docval_func(super().write, kwargs)
         if cache_spec:
             ref = self.__file.attrs.get(SPEC_LOC_ATTR)
             spec_group = None
@@ -243,7 +293,7 @@ class HDF5IO(HDMFIO):
             raise UnsupportedOperation("Cannot read from file %s in mode '%s'. Please use mode 'r', 'r+', or 'a'."
                                        % (self.__path, self.__mode))
         try:
-            return call_docval_func(super(HDF5IO, self).read, kwargs)
+            return call_docval_func(super().read, kwargs)
         except UnsupportedOperation as e:
             if str(e) == 'Cannot build data. There are no values.':
                 raise UnsupportedOperation("Cannot read data from file %s in mode '%s'. There are no values."
@@ -262,26 +312,62 @@ class HDF5IO(HDMFIO):
             self.__read[self.__file] = f_builder
         return f_builder
 
-    def __set_built(self, fpath, path, builder):
-        self.__built.setdefault(fpath, dict()).setdefault(path, builder)
+    def __set_built(self, fpath, id, builder):
+        """
+        Update self.__built to cache the given builder for the given file and id.
+
+        :param fpath: Path to the HDF5 file containing the object
+        :type fpath: str
+        :param id: ID of the HDF5 object in the path
+        :type id: h5py GroupID object
+        :param builder: The builder to be cached
+        """
+        self.__built.setdefault(fpath, dict()).setdefault(id, builder)
+
+    def __get_built(self, fpath, id):
+        """
+        Look up a builder for the given file and id in self.__built cache
+
+        :param fpath: Path to the HDF5 file containing the object
+        :type fpath: str
+        :param id: ID of the HDF5 object in the path
+        :type id: h5py GroupID object
+
+        :return: Builder in the self.__built cache or None
+        """
 
-    def __get_built(self, fpath, path):
         fdict = self.__built.get(fpath)
         if fdict:
-            return fdict.get(path)
+            return fdict.get(id)
         else:
             return None
 
     @docval({'name': 'h5obj', 'type': (Dataset, Group),
-             'doc': 'the HDF5 object to the corresponding Container/Data object for'})
-    def get_container(self, **kwargs):
+             'doc': 'the HDF5 object to the corresponding Builder object for'})
+    def get_builder(self, **kwargs):
+        """
+        Get the builder for the corresponding h5py Group or Dataset
+
+        :raises ValueError: When no builder has been constructed yet for the given h5py object
+        """
         h5obj = getargs('h5obj', kwargs)
         fpath = h5obj.file.filename
-        path = h5obj.name
-        builder = self.__get_built(fpath, path)
+        builder = self.__get_built(fpath, h5obj.id)
         if builder is None:
-            msg = '%s:%s has not been built' % (fpath, path)
+            msg = '%s:%s has not been built' % (fpath, h5obj.name)
             raise ValueError(msg)
+        return builder
+
+    @docval({'name': 'h5obj', 'type': (Dataset, Group),
+             'doc': 'the HDF5 object to the corresponding Container/Data object for'})
+    def get_container(self, **kwargs):
+        """
+        Get the container for the corresponding h5py Group or Dataset
+
+        :raises ValueError: When no builder has been constructed yet for the given h5py object
+        """
+        h5obj = getargs('h5obj', kwargs)
+        builder = self.get_builder(h5obj)
         container = self.manager.construct(builder)
         return container
 
@@ -312,20 +398,20 @@ class HDF5IO(HDMFIO):
                     builder_name = os.path.basename(target_path)
                     parent_loc = os.path.dirname(target_path)
                     # get builder if already read, else build it
-                    builder = self.__get_built(sub_h5obj.file.filename, target_path)
+                    builder = self.__get_built(sub_h5obj.file.filename, sub_h5obj.file[target_path].id)
                     if builder is None:
                         # NOTE: all links must have absolute paths
                         if isinstance(sub_h5obj, Dataset):
                             builder = self.__read_dataset(sub_h5obj, builder_name)
                         else:
                             builder = self.__read_group(sub_h5obj, builder_name, ignore=ignore)
-                        self.__set_built(sub_h5obj.file.filename, target_path, builder)
+                        self.__set_built(sub_h5obj.file.filename,  sub_h5obj.file[target_path].id, builder)
                     builder.location = parent_loc
                     link_builder = LinkBuilder(builder, k, source=h5obj.file.filename)
                     link_builder.written = True
                     kwargs['links'][builder_name] = link_builder
                 else:
-                    builder = self.__get_built(sub_h5obj.file.filename, sub_h5obj.name)
+                    builder = self.__get_built(sub_h5obj.file.filename, sub_h5obj.id)
                     obj_type = None
                     read_method = None
                     if isinstance(sub_h5obj, Dataset):
@@ -336,10 +422,10 @@ class HDF5IO(HDMFIO):
                         obj_type = kwargs['groups']
                     if builder is None:
                         builder = read_method(sub_h5obj)
-                        self.__set_built(sub_h5obj.file.filename, sub_h5obj.name, builder)
+                        self.__set_built(sub_h5obj.file.filename, sub_h5obj.id, builder)
                     obj_type[builder.name] = builder
             else:
-                warnings.warn('Broken Link: %s' % os.path.join(h5obj.name, k))
+                warnings.warn(os.path.join(h5obj.name, k), BrokenLinkWarning)
                 kwargs['datasets'][k] = None
                 continue
         kwargs['source'] = h5obj.file.filename
@@ -370,7 +456,7 @@ class HDF5IO(HDMFIO):
                 # TODO (AJTRITT):  This should call __read_ref to support Group references
                 target = h5obj.file[scalar]
                 target_builder = self.__read_dataset(target)
-                self.__set_built(target.file.filename, target.name, target_builder)
+                self.__set_built(target.file.filename, target.id, target_builder)
                 if isinstance(scalar, RegionReference):
                     kwargs['data'] = RegionBuilder(scalar, target_builder)
                 else:
@@ -381,16 +467,16 @@ class HDF5IO(HDMFIO):
             d = None
             if h5obj.dtype.kind == 'O':
                 elem1 = h5obj[0]
-                if isinstance(elem1, (text_type, binary_type)):
+                if isinstance(elem1, (str, bytes)):
                     d = h5obj
                 elif isinstance(elem1, RegionReference):  # read list of references
-                    d = H5RegionDataset(h5obj, self)
+                    d = BuilderH5RegionDataset(h5obj, self)
                 elif isinstance(elem1, Reference):
-                    d = H5ReferenceDataset(h5obj, self)
+                    d = BuilderH5ReferenceDataset(h5obj, self)
             elif h5obj.dtype.kind == 'V':    # table
                 cpd_dt = h5obj.dtype
                 ref_cols = [check_dtype(ref=cpd_dt[i]) for i in range(len(cpd_dt))]
-                d = H5TableDataset(h5obj, self, ref_cols)
+                d = BuilderH5TableDataset(h5obj, self, ref_cols)
             else:
                 d = h5obj
             kwargs["data"] = d
@@ -415,7 +501,7 @@ class HDF5IO(HDMFIO):
 
     def __read_ref(self, h5obj):
         ret = None
-        ret = self.__get_built(h5obj.file.filename, h5obj.name)
+        ret = self.__get_built(h5obj.file.filename, h5obj.id)
         if ret is None:
             if isinstance(h5obj, Dataset):
                 ret = self.__read_dataset(h5obj)
@@ -423,7 +509,7 @@ class HDF5IO(HDMFIO):
                 ret = self.__read_group(h5obj)
             else:
                 raise ValueError("h5obj must be a Dataset or a Group - got %s" % str(h5obj))
-            self.__set_built(h5obj.file.filename, h5obj.name, ret)
+            self.__set_built(h5obj.file.filename, h5obj.id, ret)
         return ret
 
     def open(self):
@@ -441,17 +527,20 @@ class HDF5IO(HDMFIO):
 
     @docval({'name': 'builder', 'type': GroupBuilder, 'doc': 'the GroupBuilder object representing the HDF5 file'},
             {'name': 'link_data', 'type': bool,
-             'doc': 'If not specified otherwise link (True) or copy (False) HDF5 Datasets', 'default': True})
+             'doc': 'If not specified otherwise link (True) or copy (False) HDF5 Datasets', 'default': True},
+            {'name': 'exhaust_dci', 'type': bool,
+             'doc': 'exhaust DataChunkIterators one at a time. If False, exhaust them concurrently', 'default': True})
     def write_builder(self, **kwargs):
-        f_builder, link_data = getargs('builder', 'link_data', kwargs)
+        f_builder, link_data, exhaust_dci = getargs('builder', 'link_data', 'exhaust_dci', kwargs)
         for name, gbldr in f_builder.groups.items():
-            self.write_group(self.__file, gbldr)
+            self.write_group(self.__file, gbldr, exhaust_dci=exhaust_dci)
         for name, dbldr in f_builder.datasets.items():
-            self.write_dataset(self.__file, dbldr, link_data)
+            self.write_dataset(self.__file, dbldr, link_data, exhaust_dci=exhaust_dci)
         for name, lbldr in f_builder.links.items():
             self.write_link(self.__file, lbldr)
         self.set_attributes(self.__file, f_builder.attributes)
         self.__add_refs()
+        self.__exhaust_dcis()
 
     def __add_refs(self):
         '''
@@ -473,9 +562,18 @@ class HDF5IO(HDMFIO):
                 failed.add(id(call))
                 self.__ref_queue.append(call)
 
+    def __exhaust_dcis(self):
+        """
+        Read and write from any queued DataChunkIterators in a round-robin fashion
+        """
+        while len(self.__dci_queue) > 0:
+            dset, data = self.__dci_queue.popleft()
+            if self.__write_chunk__(dset, data):
+                self.__dci_queue.append((dset, data))
+
     @classmethod
     def get_type(cls, data):
-        if isinstance(data, (text_type, string_types)):
+        if isinstance(data, str):
             return H5_TEXT
         elif isinstance(data, Container):
             return H5_REF
@@ -549,9 +647,9 @@ class HDF5IO(HDMFIO):
             if isinstance(value, (set, list, tuple)):
                 tmp = tuple(value)
                 if len(tmp) > 0:
-                    if isinstance(tmp[0], text_type):
+                    if isinstance(tmp[0], str):
                         value = [np.unicode_(s) for s in tmp]
-                    elif isinstance(tmp[0], binary_type):
+                    elif isinstance(tmp[0], bytes):
                         value = [np.string_(s) for s in tmp]
                     elif isinstance(tmp[0], Container):  # a list of references
                         self.__queue_ref(self._make_attr_ref_filler(obj, key, tmp))
@@ -580,10 +678,12 @@ class HDF5IO(HDMFIO):
 
     @docval({'name': 'parent', 'type': Group, 'doc': 'the parent HDF5 object'},
             {'name': 'builder', 'type': GroupBuilder, 'doc': 'the GroupBuilder to write'},
+            {'name': 'exhaust_dci', 'type': bool,
+             'doc': 'exhaust DataChunkIterators one at a time. If False, exhaust them concurrently', 'default': True},
             returns='the Group that was created', rtype='Group')
     def write_group(self, **kwargs):
 
-        parent, builder = getargs('parent', 'builder', kwargs)
+        parent, builder, exhaust_dci = getargs('parent', 'builder', 'exhaust_dci', kwargs)
         if builder.written:
             group = parent[builder.name]
         else:
@@ -593,12 +693,12 @@ class HDF5IO(HDMFIO):
         if subgroups:
             for subgroup_name, sub_builder in subgroups.items():
                 # do not create an empty group without attributes or links
-                self.write_group(group, sub_builder)
+                self.write_group(group, sub_builder, exhaust_dci=exhaust_dci)
         # write all datasets
         datasets = builder.datasets
         if datasets:
             for dset_name, sub_builder in datasets.items():
-                self.write_dataset(group, sub_builder)
+                self.write_dataset(group, sub_builder, exhaust_dci=exhaust_dci)
         # write all links
         links = builder.links
         if links:
@@ -646,18 +746,20 @@ class HDF5IO(HDMFIO):
         builder.written = True
         return link_obj
 
-    @docval({'name': 'parent', 'type': Group, 'doc': 'the parent HDF5 object'},  # noqa
+    @docval({'name': 'parent', 'type': Group, 'doc': 'the parent HDF5 object'},
             {'name': 'builder', 'type': DatasetBuilder, 'doc': 'the DatasetBuilder to write'},
             {'name': 'link_data', 'type': bool,
              'doc': 'If not specified otherwise link (True) or copy (False) HDF5 Datasets', 'default': True},
+            {'name': 'exhaust_dci', 'type': bool,
+             'doc': 'exhaust DataChunkIterators one at a time. If False, exhaust them concurrently', 'default': True},
             returns='the Dataset that was created', rtype=Dataset)
-    def write_dataset(self, **kwargs):
+    def write_dataset(self, **kwargs):  # noqa: C901
         """ Write a dataset to HDF5
 
         The function uses other dataset-dependent write functions, e.g,
-        __scalar_fill__, __list_fill__ and __chunked_iter_fill__ to write the data.
+        __scalar_fill__, __list_fill__ and __setup_chunked_dset__ to write the data.
         """
-        parent, builder, link_data = getargs('parent', 'builder', 'link_data', kwargs)
+        parent, builder, link_data, exhaust_dci = getargs('parent', 'builder', 'link_data', 'exhaust_dci', kwargs)
         if builder.written:
             return None
         name = builder.name
@@ -708,7 +810,7 @@ class HDF5IO(HDMFIO):
                     _dtype = self.__resolve_dtype__(options['dtype'], data)
                 except Exception as exc:
                     msg = 'cannot add %s to %s - could not determine type' % (name, parent.name)
-                    raise_from(Exception(msg), exc)
+                    raise Exception(msg) from exc
                 dset = parent.require_dataset(name, shape=(len(data),), dtype=_dtype, **options['io_settings'])
                 builder.written = True
 
@@ -785,11 +887,12 @@ class HDF5IO(HDMFIO):
         # write a "regular" dataset
         else:
             # Write a scalar dataset containing a single string
-            if isinstance(data, (text_type, binary_type)):
+            if isinstance(data, (str, bytes)):
                 dset = self.__scalar_fill__(parent, name, data, options)
             # Iterative write of a data chunk iterator
             elif isinstance(data, AbstractDataChunkIterator):
-                dset = self.__chunked_iter_fill__(parent, name, data, options)
+                dset = self.__setup_chunked_dset__(parent, name, data, options)
+                self.__dci_queue.append((dset, data))
             # Write a regular in memory array (e.g., numpy array, list etc.)
             elif hasattr(data, '__len__'):
                 dset = self.__list_fill__(parent, name, data, options)
@@ -803,6 +906,8 @@ class HDF5IO(HDMFIO):
         elif len(attributes) > 0:
             pass
         builder.written = True
+        if exhaust_dci:
+            self.__exhaust_dcis()
         return
 
     @classmethod
@@ -829,18 +934,18 @@ class HDF5IO(HDMFIO):
                 dtype = cls.__resolve_dtype__(dtype, data)
             except Exception as exc:
                 msg = 'cannot add %s to %s - could not determine type' % (name, parent.name)
-                raise_from(Exception(msg), exc)
+                raise Exception(msg) from exc
         try:
             dset = parent.create_dataset(name, data=data, shape=None, dtype=dtype, **io_settings)
         except Exception as exc:
             msg = "Could not create scalar dataset %s in %s" % (name, parent.name)
-            raise_from(Exception(msg), exc)
+            raise Exception(msg) from exc
         return dset
 
     @classmethod
-    def __chunked_iter_fill__(cls, parent, name, data, options=None):
+    def __setup_chunked_dset__(cls, parent, name, data, options=None):
         """
-        Write data to a dataset one-chunk-at-a-time based on the given DataChunkIterator
+        Setup a dataset for writing to one-chunk-at-a-time based on the given DataChunkIterator
 
         :param parent: The parent object to which the dataset should be added
         :type parent: h5py.Group, h5py.File
@@ -849,7 +954,7 @@ class HDF5IO(HDMFIO):
         :param data: The data to be written.
         :type data: DataChunkIterator
         :param options: Dict with options for creating a dataset. available options are 'dtype' and 'io_settings'
-        :type data: dict
+        :type options: dict
 
         """
         io_settings = {}
@@ -874,8 +979,24 @@ class HDF5IO(HDMFIO):
         try:
             dset = parent.create_dataset(name, **io_settings)
         except Exception as exc:
-            raise_from(Exception("Could not create dataset %s in %s" % (name, parent.name)), exc)
-        for chunk_i in data:
+            raise Exception("Could not create dataset %s in %s" % (name, parent.name)) from exc
+        return dset
+
+    @classmethod
+    def __write_chunk__(cls, dset, data):
+        """
+        Read a chunk from the given DataChunkIterator and write it to the given Dataset
+
+        :param dset: The Dataset to write to
+        :type dset: Dataset
+        :param data: The DataChunkIterator to read from
+        :type data: DataChunkIterator
+        :return: True of a chunk was written, False otherwise
+        :rtype: bool
+
+        """
+        try:
+            chunk_i = next(data)
             # Determine the minimum array dimensions to fit the chunk selection
             max_bounds = cls.__selection_max_bounds__(chunk_i.selection)
             if not hasattr(max_bounds, '__len__'):
@@ -889,6 +1010,29 @@ class HDF5IO(HDMFIO):
                 dset.resize(new_shape)
             # Process and write the data
             dset[chunk_i.selection] = chunk_i.data
+        except StopIteration:
+            return False
+        return True
+
+    @classmethod
+    def __chunked_iter_fill__(cls, parent, name, data, options=None):
+        """
+        Write data to a dataset one-chunk-at-a-time based on the given DataChunkIterator
+
+        :param parent: The parent object to which the dataset should be added
+        :type parent: h5py.Group, h5py.File
+        :param name: The name of the dataset
+        :type name: str
+        :param data: The data to be written.
+        :type data: DataChunkIterator
+        :param options: Dict with options for creating a dataset. available options are 'dtype' and 'io_settings'
+        :type options: dict
+
+        """
+        dset = cls.__setup_chunked_dset__(parent, name, data, options=options)
+        read = True
+        while read:
+            read = cls.__write_chunk__(dset, data)
         return dset
 
     @classmethod
@@ -904,7 +1048,7 @@ class HDF5IO(HDMFIO):
                 dtype = cls.__resolve_dtype__(dtype, data)
             except Exception as exc:
                 msg = 'cannot add %s to %s - could not determine type' % (name, parent.name)
-                raise_from(Exception(msg), exc)
+                raise Exception(msg) from exc
         # define the data shape
         if 'shape' in io_settings:
             data_shape = io_settings.pop('shape')
@@ -913,14 +1057,14 @@ class HDF5IO(HDMFIO):
         elif isinstance(dtype, np.dtype):
             data_shape = (len(data),)
         else:
-            data_shape = get_shape(data)
+            data_shape = get_data_shape(data)
         # Create the dataset
         try:
             dset = parent.create_dataset(name, shape=data_shape, dtype=dtype, **io_settings)
         except Exception as exc:
             msg = "Could not create dataset %s in %s with shape %s, dtype %s, and iosettings %s. %s" % \
                   (name, parent.name, str(data_shape), str(dtype), str(io_settings), str(exc))
-            raise_from(Exception(msg), exc)
+            raise Exception(msg) from exc
         # Write the data
         if len(data) > dset.shape[0]:
             new_shape = list(dset.shape)
@@ -964,10 +1108,11 @@ class HDF5IO(HDMFIO):
     def __is_ref(self, dtype):
         if isinstance(dtype, DtypeSpec):
             return self.__is_ref(dtype.dtype)
-        elif isinstance(dtype, RefSpec):
+        if isinstance(dtype, RefSpec):
             return True
-        else:
+        if isinstance(dtype, str):
             return dtype == DatasetBuilder.OBJECT_REF_TYPE or dtype == DatasetBuilder.REGION_REF_TYPE
+        return False
 
     def __queue_ref(self, func):
         '''Set aside filling dset with references
