@@ -48,7 +48,7 @@ except ImportError:
     from io import StringIO
 import sys
 import re
-import unittest2 as unittest
+import unittest
 import textwrap
 
 
@@ -204,12 +204,13 @@ class ColoredTestResult(unittest.TextTestResult):
     stderr_redirector = OutputRedirector(sys.stderr)
 
     def __init__(self, stream, descriptions, verbosity=1):
-        super(ColoredTestResult, self).__init__(stream, descriptions, verbosity)
+        super().__init__(stream, descriptions, verbosity)
         self.stdout0 = None
         self.stderr0 = None
         self.success_count = 0
         self.failure_count = 0
         self.error_count = 0
+        self.skip_count = 0
         self.verbosity = verbosity
 
         # deny TextTestResult showAll functionality
@@ -225,7 +226,7 @@ class ColoredTestResult(unittest.TextTestResult):
         self.result = []
 
     def startTest(self, test):
-        super(ColoredTestResult, self).startTest(test)
+        super().startTest(test)
         # just one buffer for both stdout and stderr
         self.outputBuffer = StringIO()
         self.stdout_redirector.fp = self.outputBuffer
@@ -255,15 +256,11 @@ class ColoredTestResult(unittest.TextTestResult):
 
     def addSuccess(self, test):
         self.success_count += 1
-        super(ColoredTestResult, self).addSuccess(test)
+        super().addSuccess(test)
         output = self.complete_output()
         self.result.append((0, test, output, ''))
-        # if self.verbosity > 1:
-        #     sys.stderr.write('ok ')
-        #     sys.stderr.write(str(test))
-        #     sys.stderr.write('\n')
-        # else:
-        #     pass #sys.stderr.write('.')
+        sys.stdout.write('.')
+        sys.stdout.flush()
 
         if not hasattr(self, 'successes'):
             self.successes = [test]
@@ -272,29 +269,35 @@ class ColoredTestResult(unittest.TextTestResult):
 
     def addError(self, test, err):
         self.error_count += 1
-        super(ColoredTestResult, self).addError(test, err)
-        _, _exc_str = self.errors[-1]
+        super().addError(test, err)
         output = self.complete_output()
+        _, _exc_str = self.errors[-1]
         self.result.append((2, test, output, _exc_str))
-        # if self.verbosity > 1:
-        #     sys.stderr.write('E  ')
-        #     sys.stderr.write(str(test))
-        #     sys.stderr.write('\n')
-        # else:
-        #     pass #sys.stderr.write('E')
+        sys.stdout.write('E')
+        sys.stdout.flush()
 
     def addFailure(self, test, err):
         self.failure_count += 1
-        super(ColoredTestResult, self).addFailure(test, err)
-        _, _exc_str = self.failures[-1]
+        super().addFailure(test, err)
         output = self.complete_output()
+        _, _exc_str = self.failures[-1]
         self.result.append((1, test, output, _exc_str))
-        # if self.verbosity > 1:
-        #     sys.stderr.write('F  ')
-        #     sys.stderr.write(str(test))
-        #     sys.stderr.write('\n')
-        # else:
-        #     pass #sys.stderr.write('F')
+        sys.stdout.write('F')
+        sys.stdout.flush()
+
+    def addSubTest(self, test, subtest, err):
+        if err is not None:
+            if issubclass(err[0], test.failureException):
+                self.addFailure(subtest, err)
+            else:
+                self.addError(subtest, err)
+
+    def addSkip(self, test, reason):
+        self.skip_count += 1
+        super().addSkip(test, reason)
+        self.complete_output()
+        sys.stdout.write('s')
+        sys.stdout.flush()
 
     def get_all_cases_run(self):
         '''Return a list of each test case which failed or succeeded
