@@ -1,28 +1,28 @@
-import unittest2 as unittest
-import re
-
-from hdmf.spec import GroupSpec, AttributeSpec, DatasetSpec, SpecCatalog, SpecNamespace, NamespaceCatalog
+from hdmf.spec import GroupSpec, AttributeSpec, DatasetSpec, SpecCatalog, SpecNamespace, NamespaceCatalog, RefSpec
 from hdmf.build import GroupBuilder, DatasetBuilder, ObjectMapper, BuildManager, TypeMap, LinkBuilder
 from hdmf import Container
 from hdmf.utils import docval, getargs, get_docval
+from hdmf.data_utils import DataChunkIterator
+from hdmf.backends.hdf5 import H5DataIO
+from hdmf.testing import TestCase
 
-from abc import ABCMeta
-from six import with_metaclass
+from abc import ABCMeta, abstractmethod
+import numpy as np
 
-from tests.unit.test_utils import CORE_NAMESPACE
+from tests.unit.utils import CORE_NAMESPACE
 
 
 class Bar(Container):
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this Bar'},
-            {'name': 'data', 'type': list, 'doc': 'some data'},
+            {'name': 'data', 'type': ('data', 'array_data'), 'doc': 'some data'},
             {'name': 'attr1', 'type': str, 'doc': 'an attribute'},
             {'name': 'attr2', 'type': int, 'doc': 'another attribute'},
             {'name': 'attr3', 'type': float, 'doc': 'a third attribute', 'default': 3.14},
             {'name': 'foo', 'type': 'Foo', 'doc': 'a group', 'default': None})
     def __init__(self, **kwargs):
         name, data, attr1, attr2, attr3, foo = getargs('name', 'data', 'attr1', 'attr2', 'attr3', 'foo', kwargs)
-        super(Bar, self).__init__(name=name)
+        super().__init__(name=name)
         self.__data = data
         self.__attr1 = attr1
         self.__attr2 = attr2
@@ -71,13 +71,15 @@ class Foo(Container):
         return 'Foo'
 
 
-class TestGetSubSpec(unittest.TestCase):
+class TestGetSubSpec(TestCase):
 
     def setUp(self):
         self.bar_spec = GroupSpec('A test group specification with a data type', data_type_def='Bar')
         spec_catalog = SpecCatalog()
         spec_catalog.register_spec(self.bar_spec, 'test.yaml')
-        namespace = SpecNamespace('a test namespace', CORE_NAMESPACE, [{'source': 'test.yaml'}], catalog=spec_catalog)
+        namespace = SpecNamespace('a test namespace', CORE_NAMESPACE, [{'source': 'test.yaml'}],
+                                  version='0.1.0',
+                                  catalog=spec_catalog)
         namespace_catalog = NamespaceCatalog()
         namespace_catalog.add_namespace(CORE_NAMESPACE, namespace)
         self.type_map = TypeMap(namespace_catalog)
@@ -87,7 +89,7 @@ class TestGetSubSpec(unittest.TestCase):
         parent_spec = GroupSpec('Something to hold a Bar', 'bar_bucket', groups=[self.bar_spec])
         sub_builder = GroupBuilder('my_bar', attributes={'data_type': 'Bar', 'namespace': CORE_NAMESPACE,
                                                          'object_id': -1})
-        builder = GroupBuilder('bar_bucket', groups={'my_bar': sub_builder})  # noqa: F841
+        GroupBuilder('bar_bucket', groups={'my_bar': sub_builder})
         result = self.type_map.get_subspec(parent_spec, sub_builder)
         self.assertIs(result, self.bar_spec)
 
@@ -96,12 +98,12 @@ class TestGetSubSpec(unittest.TestCase):
         parent_spec = GroupSpec('Something to hold a Bar', 'my_group', groups=[child_spec])
         sub_builder = GroupBuilder('my_subgroup', attributes={'data_type': 'Bar', 'namespace': CORE_NAMESPACE,
                                                               'object_id': -1})
-        builder = GroupBuilder('my_group', groups={'my_bar': sub_builder})  # noqa: F841
+        GroupBuilder('my_group', groups={'my_bar': sub_builder})
         result = self.type_map.get_subspec(parent_spec, sub_builder)
         self.assertIs(result, child_spec)
 
 
-class TestTypeMap(unittest.TestCase):
+class TestTypeMap(TestCase):
 
     def setUp(self):
         self.bar_spec = GroupSpec('A test group specification with a data type', data_type_def='Bar')
@@ -110,6 +112,7 @@ class TestTypeMap(unittest.TestCase):
         self.spec_catalog.register_spec(self.bar_spec, 'test.yaml')
         self.spec_catalog.register_spec(self.foo_spec, 'test.yaml')
         self.namespace = SpecNamespace('a test namespace', CORE_NAMESPACE, [{'source': 'test.yaml'}],
+                                       version='0.1.0',
                                        catalog=self.spec_catalog)
         self.namespace_catalog = NamespaceCatalog()
         self.namespace_catalog.add_namespace(CORE_NAMESPACE, self.namespace)
@@ -147,7 +150,67 @@ class TestTypeMap(unittest.TestCase):
         self.assertIsInstance(mapper, MyMap)
 
 
-class TestDynamicContainer(unittest.TestCase):
+class BarMapper(ObjectMapper):
+    def __init__(self, spec):
+        super().__init__(spec)
+        data_spec = spec.get_dataset('data')
+        self.map_spec('attr2', data_spec.get_attribute('attr2'))
+
+
+class TestMapStrings(TestCase):
+
+    def customSetUp(self, bar_spec):
+        spec_catalog = SpecCatalog()
+        spec_catalog.register_spec(bar_spec, 'test.yaml')
+        namespace = SpecNamespace('a test namespace', CORE_NAMESPACE, [{'source': 'test.yaml'}], version='0.1.0',
+                                  catalog=spec_catalog)
+        namespace_catalog = NamespaceCatalog()
+        namespace_catalog.add_namespace(CORE_NAMESPACE, namespace)
+        type_map = TypeMap(namespace_catalog)
+        type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
+        return type_map
+
+    def test_build_1d(self):
+        bar_spec = GroupSpec('A test group specification with a data type',
+                             data_type_def='Bar',
+                             datasets=[DatasetSpec('an example dataset', 'text', name='data', shape=(None,),
+                                                   attributes=[AttributeSpec(
+                                                       'attr2', 'an example integer attribute', 'int')])],
+                             attributes=[AttributeSpec('attr1', 'an example string attribute', 'text')])
+        type_map = self.customSetUp(bar_spec)
+        type_map.register_map(Bar, BarMapper)
+        bar_inst = Bar('my_bar', ['a', 'b', 'c', 'd'], 'value1', 10)
+        builder = type_map.build(bar_inst)
+        self.assertEqual(builder.get('data').data, ['a', 'b', 'c', 'd'])
+
+    def test_build_scalar(self):
+        bar_spec = GroupSpec('A test group specification with a data type',
+                             data_type_def='Bar',
+                             datasets=[DatasetSpec('an example dataset', 'text', name='data',
+                                                   attributes=[AttributeSpec(
+                                                       'attr2', 'an example integer attribute', 'int')])],
+                             attributes=[AttributeSpec('attr1', 'an example string attribute', 'text')])
+        type_map = self.customSetUp(bar_spec)
+        type_map.register_map(Bar, BarMapper)
+        bar_inst = Bar('my_bar', ['a', 'b', 'c', 'd'], 'value1', 10)
+        builder = type_map.build(bar_inst)
+        self.assertEqual(builder.get('data').data, "['a', 'b', 'c', 'd']")
+
+    def test_build_dataio(self):
+        bar_spec = GroupSpec('A test group specification with a data type',
+                             data_type_def='Bar',
+                             datasets=[DatasetSpec('an example dataset', 'text', name='data', shape=(None,),
+                                                   attributes=[AttributeSpec(
+                                                       'attr2', 'an example integer attribute', 'int')])],
+                             attributes=[AttributeSpec('attr1', 'an example string attribute', 'text')])
+        type_map = self.customSetUp(bar_spec)
+        type_map.register_map(Bar, BarMapper)
+        bar_inst = Bar('my_bar', H5DataIO(['a', 'b', 'c', 'd'], chunks=True), 'value1', 10)
+        builder = type_map.build(bar_inst)
+        self.assertIsInstance(builder.get('data').data, H5DataIO)
+
+
+class TestDynamicContainer(TestCase):
 
     def setUp(self):
         self.bar_spec = GroupSpec('A test group specification with a data type',
@@ -159,7 +222,9 @@ class TestDynamicContainer(unittest.TestCase):
         self.spec_catalog = SpecCatalog()
         self.spec_catalog.register_spec(self.bar_spec, 'test.yaml')
         self.namespace = SpecNamespace('a test namespace', CORE_NAMESPACE,
-                                       [{'source': 'test.yaml'}], catalog=self.spec_catalog)
+                                       [{'source': 'test.yaml'}],
+                                       version='0.1.0',
+                                       catalog=self.spec_catalog)
         self.namespace_catalog = NamespaceCatalog()
         self.namespace_catalog.add_namespace(CORE_NAMESPACE, self.namespace)
         self.type_map = TypeMap(self.namespace_catalog)
@@ -299,18 +364,20 @@ class TestDynamicContainer(unittest.TestCase):
         # Setup all the data we need
         msg = ("Cannot dynamically generate class for type 'Baz1'. Type 'Baz2' does not exist. "
                "Please define that type before defining 'Baz1'.")
-        with self.assertRaisesRegex(ValueError, re.escape(msg)):
+        with self.assertRaisesWith(ValueError, msg):
             self.manager.type_map.get_container_cls(CORE_NAMESPACE, 'Baz1')
 
 
-class TestObjectMapper(with_metaclass(ABCMeta, unittest.TestCase)):
+class ObjectMapperMixin(metaclass=ABCMeta):
 
     def setUp(self):
         self.setUpBarSpec()
         self.spec_catalog = SpecCatalog()
         self.spec_catalog.register_spec(self.bar_spec, 'test.yaml')
         self.namespace = SpecNamespace('a test namespace', CORE_NAMESPACE,
-                                       [{'source': 'test.yaml'}], catalog=self.spec_catalog)
+                                       [{'source': 'test.yaml'}],
+                                       version='0.1.0',
+                                       catalog=self.spec_catalog)
         self.namespace_catalog = NamespaceCatalog()
         self.namespace_catalog.add_namespace(CORE_NAMESPACE, self.namespace)
         self.type_map = TypeMap(self.namespace_catalog)
@@ -319,8 +386,9 @@ class TestObjectMapper(with_metaclass(ABCMeta, unittest.TestCase)):
         self.manager = BuildManager(self.type_map)
         self.mapper = ObjectMapper(self.bar_spec)
 
+    @abstractmethod
     def setUpBarSpec(self):
-        raise unittest.SkipTest('setUpBarSpec not implemented')
+        raise NotImplementedError('Cannot run test unless setUpBarSpec is implemented')
 
     def test_default_mapping(self):
         attr_map = self.mapper.get_attr_names(self.bar_spec)
@@ -331,7 +399,7 @@ class TestObjectMapper(with_metaclass(ABCMeta, unittest.TestCase)):
                 self.assertIs(attr_map[key], self.mapper.get_carg_spec(key))
 
 
-class TestObjectMapperNested(TestObjectMapper):
+class TestObjectMapperNested(ObjectMapperMixin, TestCase):
 
     def setUpBarSpec(self):
         self.bar_spec = GroupSpec('A test group specification with a data type',
@@ -348,6 +416,7 @@ class TestObjectMapperNested(TestObjectMapper):
         expected = GroupBuilder('my_bar', datasets={'data': DatasetBuilder(
             'data', list(range(10)), attributes={'attr2': 10})},
                                 attributes={'attr1': 'value1'})
+        self._remap_nested_attr()
         builder = self.mapper.build(container_inst, self.manager)
         self.assertDictEqual(builder, expected)
 
@@ -359,17 +428,29 @@ class TestObjectMapperNested(TestObjectMapper):
             'data', list(range(10)), attributes={'attr2': 10})},
                                attributes={'attr1': 'value1', 'data_type': 'Bar', 'namespace': CORE_NAMESPACE,
                                            'object_id': expected.object_id})
+        self._remap_nested_attr()
         container = self.mapper.construct(builder, self.manager)
         self.assertEqual(container, expected)
 
     def test_default_mapping_keys(self):
         attr_map = self.mapper.get_attr_names(self.bar_spec)
         keys = set(attr_map.keys())
-        expected = {'attr1', 'data', 'attr2'}
+        expected = {'attr1', 'data', 'data__attr2'}
         self.assertSetEqual(keys, expected)
 
+    def test_remap_keys(self):
+        self._remap_nested_attr()
+        self.assertEqual(self.mapper.get_attr_spec('attr2'),
+                         self.mapper.spec.get_dataset('data').get_attribute('attr2'))
+        self.assertEqual(self.mapper.get_attr_spec('attr1'), self.mapper.spec.get_attribute('attr1'))
+        self.assertEqual(self.mapper.get_attr_spec('data'), self.mapper.spec.get_dataset('data'))
 
-class TestObjectMapperNoNesting(TestObjectMapper):
+    def _remap_nested_attr(self):
+        data_spec = self.mapper.spec.get_dataset('data')
+        self.mapper.map_spec('attr2', data_spec.get_attribute('attr2'))
+
+
+class TestObjectMapperNoNesting(ObjectMapperMixin, TestCase):
 
     def setUpBarSpec(self):
         self.bar_spec = GroupSpec('A test group specification with a data type',
@@ -409,7 +490,7 @@ class TestObjectMapperNoNesting(TestObjectMapper):
         self.assertSetEqual(keys, expected)
 
 
-class TestObjectMapperContainer(TestObjectMapper):
+class TestObjectMapperContainer(ObjectMapperMixin, TestCase):
 
     def setUpBarSpec(self):
         self.bar_spec = GroupSpec('A test group specification with a data type',
@@ -425,7 +506,7 @@ class TestObjectMapperContainer(TestObjectMapper):
         self.assertSetEqual(keys, expected)
 
 
-class TestLinkedContainer(unittest.TestCase):
+class TestLinkedContainer(TestCase):
 
     def setUp(self):
         self.foo_spec = GroupSpec('A test group specification with data type Foo', data_type_def='Foo')
@@ -440,7 +521,9 @@ class TestLinkedContainer(unittest.TestCase):
         self.spec_catalog.register_spec(self.foo_spec, 'test.yaml')
         self.spec_catalog.register_spec(self.bar_spec, 'test.yaml')
         self.namespace = SpecNamespace('a test namespace', CORE_NAMESPACE,
-                                       [{'source': 'test.yaml'}], catalog=self.spec_catalog)
+                                       [{'source': 'test.yaml'}],
+                                       version='0.1.0',
+                                       catalog=self.spec_catalog)
         self.namespace_catalog = NamespaceCatalog()
         self.namespace_catalog.add_namespace(CORE_NAMESPACE, self.namespace)
         self.type_map = TypeMap(self.namespace_catalog)
@@ -483,5 +566,161 @@ class TestLinkedContainer(unittest.TestCase):
         self.assertDictEqual(bar2_builder, bar2_expected)
 
 
-if __name__ == '__main__':
-    unittest.main()
+class TestConvertDtype(TestCase):
+
+    def test_value_none(self):
+        spec = DatasetSpec('an example dataset', 'int', name='data')
+        self.assertTupleEqual(ObjectMapper.convert_dtype(spec, None), (None, 'int'))
+
+        spec = DatasetSpec('an example dataset', RefSpec(reftype='object', target_type='int'), name='data')
+        self.assertTupleEqual(ObjectMapper.convert_dtype(spec, None), (None, 'object'))
+
+    def test_convert_higher_precision(self):
+        """Test that passing a data type with a precision <= specified returns the higher precision type"""
+        spec_type = 'float64'
+        value_types = ['float', 'float32', 'double', 'float64']
+        self.convert_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'int64'
+        value_types = ['long', 'int64', 'uint64', 'int', 'int32', 'int16', 'int8']
+        self.convert_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'int32'
+        value_types = ['int32', 'int16', 'int8']
+        self.convert_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'int16'
+        value_types = ['int16', 'int8']
+        self.convert_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'uint32'
+        value_types = ['uint32', 'uint16', 'uint8']
+        self.convert_higher_precision_helper(spec_type, value_types)
+
+    def convert_higher_precision_helper(self, spec_type, value_types):
+        data = 2
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+        match = (np.dtype(spec_type).type(data), np.dtype(spec_type))
+        for dtype in value_types:
+            value = np.dtype(dtype).type(data)
+            with self.subTest(dtype=dtype):
+                ret = ObjectMapper.convert_dtype(spec, value)
+                self.assertTupleEqual(ret, match)
+                self.assertIs(ret[0].dtype, match[1])
+
+    def test_keep_higher_precision(self):
+        """Test that passing a data type with a precision >= specified return the given type"""
+        spec_type = 'float'
+        value_types = ['double', 'float64']
+        self.keep_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'int'
+        value_types = ['int64']
+        self.keep_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'int8'
+        value_types = ['long', 'int64', 'int', 'int32', 'int16']
+        self.keep_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'uint'
+        value_types = ['uint64']
+        self.keep_higher_precision_helper(spec_type, value_types)
+
+        spec_type = 'uint8'
+        value_types = ['uint64', 'uint32', 'uint', 'uint16']
+        self.keep_higher_precision_helper(spec_type, value_types)
+
+    def keep_higher_precision_helper(self, spec_type, value_types):
+        data = 2
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+        for dtype in value_types:
+            value = np.dtype(dtype).type(data)
+            match = (value, np.dtype(dtype))
+            with self.subTest(dtype=dtype):
+                ret = ObjectMapper.convert_dtype(spec, value)
+                self.assertTupleEqual(ret, match)
+                self.assertIs(ret[0].dtype, match[1])
+
+    def test_no_spec(self):
+        spec_type = None
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+
+        value = [1, 2, 3]
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, int)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0][0]), match[1])
+
+        value = np.uint64(4)
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.uint64)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0]), match[1])
+
+        value = 'hello'
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, 'utf8')
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0]), str)
+
+        value = bytes('hello', encoding='utf-8')
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, 'ascii')
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0]), bytes)
+
+        value = DataChunkIterator(data=[1, 2, 3])
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.dtype(int).type)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(ret[0].dtype.type, match[1])
+
+        value = DataChunkIterator(data=[1., 2., 3.])
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.dtype(float).type)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(ret[0].dtype.type, match[1])
+
+        value = H5DataIO(np.arange(30).reshape(5, 2, 3))
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.dtype(int).type)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(ret[0].dtype.type, match[1])
+
+        value = H5DataIO(['foo' 'bar'])
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, 'utf8')
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0].data[0]), str)
+
+    def test_numeric_spec(self):
+        spec_type = 'numeric'
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+
+        value = np.uint64(4)
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.uint64)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0]), match[1])
+
+        value = DataChunkIterator(data=[1, 2, 3])
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.dtype(int).type)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(ret[0].dtype.type, match[1])
+
+    def test_bool_spec(self):
+        spec_type = 'bool'
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+
+        value = np.bool_(True)
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.bool_)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0]), match[1])
+
+        value = True
+        ret = ObjectMapper.convert_dtype(spec, value)
+        match = (value, np.bool_)
+        self.assertTupleEqual(ret, match)
+        self.assertIs(type(ret[0]), match[1])
