@@ -1,7 +1,7 @@
-import unittest2 as unittest
-from six import text_type
+import numpy as np
 
 from hdmf.utils import docval, fmt_docval_args, get_docval, popargs
+from hdmf.testing import TestCase
 
 
 class MyTestClass(object):
@@ -15,11 +15,6 @@ class MyTestClass(object):
     def basic_add2(self, **kwargs):
         return kwargs
 
-    @docval({'name': 'arg1', 'type': text_type, 'doc': 'argument1 is a str'},  # noqa: F40
-            {'name': 'arg2', 'type': int, 'doc': 'argument2 is a int'})
-    def basic_add2_text_type(self, **kwargs):
-        return kwargs
-
     @docval({'name': 'arg1', 'type': str, 'doc': 'argument1 is a str'},
             {'name': 'arg2', 'type': 'int', 'doc': 'argument2 is a int'},
             {'name': 'arg3', 'type': bool, 'doc': 'argument3 is a bool. it defaults to False', 'default': False})
@@ -31,6 +26,13 @@ class MyTestClass(object):
     def basic_only_kw(self, **kwargs):
         return kwargs
 
+    @docval({'name': 'arg1', 'type': str, 'doc': 'argument1 is a str'},
+            {'name': 'arg2', 'type': 'int', 'doc': 'argument2 is a int'},
+            {'name': 'arg3', 'type': bool, 'doc': 'argument3 is a bool. it defaults to False', 'default': False},
+            allow_extra=True)
+    def basic_add2_kw_allow_extra(self, **kwargs):
+        return kwargs
+
 
 class MyTestSubclass(MyTestClass):
 
@@ -98,15 +100,15 @@ class MyChainClass(MyTestClass):
         self._arg4 = val
 
 
-class TestDocValidator(unittest.TestCase):
+class TestDocValidator(TestCase):
 
     def setUp(self):
         self.test_obj = MyTestClass()
         self.test_obj_sub = MyTestSubclass()
 
     def test_bad_type(self):
-        exp_msg = ("error parsing 'arg1' argument' : argtype must be a type, "
-                   "a str, a list, a tuple, or None - got <class|type 'dict'>")
+        exp_msg = (r"error parsing 'arg1' argument' : argtype must be a type, "
+                   r"a str, a list, a tuple, or None - got <class|type 'dict'>")
         with self.assertRaisesRegex(Exception, exp_msg):
             @docval({'name': 'arg1', 'type': {'a': 1}, 'doc': 'this is a bad type'})
             def method(self, **kwargs):
@@ -148,6 +150,29 @@ class TestDocValidator(unittest.TestCase):
         exp_kwargs = {'arg3': True}
         self.assertDictEqual(rec_kwargs, exp_kwargs)
 
+    def test_fmt_docval_args_no_docval(self):
+        """ Test that fmt_docval_args raises an error when run on function without docval """
+        def method1(self, **kwargs):
+            pass
+
+        with self.assertRaisesRegex(ValueError, r"no docval found on .*method1.*"):
+            fmt_docval_args(method1, {})
+
+    def test_fmt_docval_args_allow_extra(self):
+        """ Test that fmt_docval_args works """
+        test_kwargs = {
+            'arg1': 'a string',
+            'arg2': 1,
+            'arg3': True,
+            'hello': 'abc',
+            'list': ['abc', 1, 2, 3]
+        }
+        rec_args, rec_kwargs = fmt_docval_args(self.test_obj.basic_add2_kw_allow_extra, test_kwargs)
+        exp_args = ['a string', 1]
+        self.assertListEqual(rec_args, exp_args)
+        exp_kwargs = {'arg3': True, 'hello': 'abc', 'list': ['abc', 1, 2, 3]}
+        self.assertDictEqual(rec_kwargs, exp_kwargs)
+
     def test_docval_add(self):
         """Test that docval works with a single positional
            argument
@@ -166,8 +191,8 @@ class TestDocValidator(unittest.TestCase):
         """Test that docval catches missing argument
            with a single positional argument
         """
-        with self.assertRaisesRegex(TypeError, "missing argument 'arg1'"):
-            kwargs = self.test_obj.basic_add()  # noqa: F841
+        with self.assertRaisesWith(TypeError, "MyTestClass.basic_add: missing argument 'arg1'"):
+            self.test_obj.basic_add()
 
     def test_docval_add2(self):
         """Test that docval works with two positional
@@ -176,18 +201,11 @@ class TestDocValidator(unittest.TestCase):
         kwargs = self.test_obj.basic_add2('a string', 100)
         self.assertDictEqual(kwargs, {'arg1': 'a string', 'arg2': 100})
 
-    def test_docval_add2_text_type_w_str(self):
+    def test_docval_add2_w_unicode(self):
         """Test that docval works with two positional
            arguments
         """
-        kwargs = self.test_obj.basic_add2_text_type('a string', 100)
-        self.assertDictEqual(kwargs, {'arg1': 'a string', 'arg2': 100})
-
-    def test_docval_add2_text_type_w_unicode(self):
-        """Test that docval works with two positional
-           arguments
-        """
-        kwargs = self.test_obj.basic_add2_text_type(u'a string', 100)
+        kwargs = self.test_obj.basic_add2(u'a string', 100)
         self.assertDictEqual(kwargs, {'arg1': u'a string', 'arg2': 100})
 
     def test_docval_add2_kw_default(self):
@@ -236,8 +254,9 @@ class TestDocValidator(unittest.TestCase):
            arguments and a keyword argument when specifying
            keyword argument value with positional syntax
         """
-        with self.assertRaisesRegex(TypeError, r"incorrect type for 'arg2' \(got 'str', expected 'int'\)"):
-            kwargs = self.test_obj.basic_add2_kw('a string', 'bad string')  # noqa: F841
+        msg = "MyTestClass.basic_add2_kw: incorrect type for 'arg2' (got 'str', expected 'int')"
+        with self.assertRaisesWith(TypeError, msg):
+            self.test_obj.basic_add2_kw('a string', 'bad string')
 
     def test_docval_add_sub(self):
         """Test that docval works with a two positional arguments,
@@ -264,8 +283,8 @@ class TestDocValidator(unittest.TestCase):
            argument is specified in both the parent and sublcass implementations,
            when using default values for keyword arguments
         """
-        with self.assertRaisesRegex(TypeError, "missing argument 'arg5'"):
-            kwargs = self.test_obj_sub.basic_add2_kw('a string', 100, 'another string')  # noqa: F841
+        with self.assertRaisesWith(TypeError, "MyTestSubclass.basic_add2_kw: missing argument 'arg5'"):
+            self.test_obj_sub.basic_add2_kw('a string', 100, 'another string')
 
     def test_docval_add2_kw_kwsyntax_sub(self):
         """Test that docval works when called with a four positional
@@ -284,16 +303,17 @@ class TestDocValidator(unittest.TestCase):
            arguments and two keyword arguments, where two positional and one keyword
            argument is specified in both the parent and sublcass implementations
         """
-        with self.assertRaisesRegex(TypeError, "missing argument 'arg5'"):
-            kwargs = self.test_obj_sub.basic_add2_kw('a string', 100, 'another string', arg6=True)  # noqa: F841
+        with self.assertRaisesWith(TypeError, "MyTestSubclass.basic_add2_kw: missing argument 'arg5'"):
+            self.test_obj_sub.basic_add2_kw('a string', 100, 'another string', arg6=True)
 
     def test_docval_add2_kw_kwsyntax_sub_nonetype_arg(self):
         """Test that docval catches NoneType when called with a four positional
            arguments and two keyword arguments, where two positional and one keyword
            argument is specified in both the parent and sublcass implementations
         """
-        with self.assertRaisesRegex(TypeError, r"None is not allowed for 'arg5' \(expected 'float', not None\)"):
-            kwargs = self.test_obj_sub.basic_add2_kw('a string', 100, 'another string', None, arg6=True)  # noqa: F841
+        msg = "MyTestSubclass.basic_add2_kw: None is not allowed for 'arg5' (expected 'float', not None)"
+        with self.assertRaisesWith(TypeError, msg):
+            self.test_obj_sub.basic_add2_kw('a string', 100, 'another string', None, arg6=True)
 
     def test_only_kw_no_args(self):
         """Test that docval parses arguments when only keyword
@@ -350,6 +370,66 @@ class TestDocValidator(unittest.TestCase):
         with self.assertRaises(TypeError):
             self.test_obj.basic_add2_kw('a string', 100, bar=1000)
 
+    def test_extra_args_pos_only(self):
+        """Test that docval raises an error if too many positional
+           arguments are specified
+        """
+        msg = ("MyTestClass.basic_add2_kw: Expected at most 3 arguments ['arg1', 'arg2', 'arg3'], got 4: 4 positional "
+               "and 0 keyword []")
+        with self.assertRaisesWith(TypeError, msg):
+            self.test_obj.basic_add2_kw('a string', 100, True, 'extra')
+
+    def test_extra_args_pos_kw(self):
+        """Test that docval raises an error if too many positional
+           arguments are specified and a keyword arg is specified
+        """
+        msg = ("MyTestClass.basic_add2_kw: Expected at most 3 arguments ['arg1', 'arg2', 'arg3'], got 4: 3 positional "
+               "and 1 keyword ['arg3']")
+        with self.assertRaisesWith(TypeError, msg):
+            self.test_obj.basic_add2_kw('a string', 'extra', 100, arg3=True)
+
+    def test_extra_kwargs_pos_kw(self):
+        """Test that docval raises an error if extra keyword
+           arguments are specified
+        """
+        msg = ("MyTestClass.basic_add2_kw: Expected at most 3 arguments ['arg1', 'arg2', 'arg3'], got 4: 2 positional "
+               "and 2 keyword ['arg3', 'extra']")
+        with self.assertRaisesWith(TypeError, msg):
+            self.test_obj.basic_add2_kw('a string', 100, extra='extra', arg3=True)
+
+    def test_extra_args_pos_only_ok(self):
+        """Test that docval raises an error if too many positional
+           arguments are specified even if allow_extra is True
+        """
+        msg = ("MyTestClass.basic_add2_kw_allow_extra: Expected at most 3 arguments ['arg1', 'arg2', 'arg3'], got "
+               "4 positional")
+        with self.assertRaisesWith(TypeError, msg):
+            self.test_obj.basic_add2_kw_allow_extra('a string', 100, True, 'extra', extra='extra')
+
+    def test_extra_args_pos_kw_ok(self):
+        """Test that docval does not raise an error if too many
+           keyword arguments are specified and allow_extra is True
+        """
+        kwargs = self.test_obj.basic_add2_kw_allow_extra('a string', 100, True, extra='extra')
+        self.assertDictEqual(kwargs, {'arg1': 'a string', 'arg2': 100, 'arg3': True, 'extra': 'extra'})
+
+    def test_dup_kw(self):
+        """Test that docval raises an error if a keyword argument
+           captures a positional argument before all positional
+           arguments have been resolved
+        """
+        with self.assertRaisesWith(TypeError, "MyTestClass.basic_add2_kw: got multiple values for argument 'arg1'"):
+            self.test_obj.basic_add2_kw('a string', 100, arg1='extra')
+
+    def test_extra_args_dup_kw(self):
+        """Test that docval raises an error if a keyword argument
+           captures a positional argument before all positional
+           arguments have been resolved and allow_extra is True
+        """
+        msg = "MyTestClass.basic_add2_kw_allow_extra: got multiple values for argument 'arg1'"
+        with self.assertRaisesWith(TypeError, msg):
+            self.test_obj.basic_add2_kw_allow_extra('a string', 100, True, arg1='extra')
+
     def test_unsupported_docval_term(self):
         """Test that docval does not allow setting of arguments
            marked as unsupported
@@ -360,14 +440,15 @@ class TestDocValidator(unittest.TestCase):
         with self.assertRaises(ValueError):
             method(self, arg1=[[1, 1]])
 
-    def test_catch_duplicate_names(self):
+    def test_catch_dup_names(self):
         """Test that docval does not allow duplicate argument names
         """
         @docval({'name': 'arg1', 'type': 'array_data', 'doc': 'this is a bad shape'},
                 {'name': 'arg1', 'type': 'array_data', 'doc': 'this is a bad shape2'})
         def method(self, **kwargs):
             pass
-        with self.assertRaisesRegex(ValueError, r"The following names are duplicated: \['arg1'\]"):
+        msg = "TestDocValidator.test_catch_dup_names.<locals>.method: The following names are duplicated: ['arg1']"
+        with self.assertRaisesWith(ValueError, msg):
             method(self, arg1=[1])
 
     def test_get_docval_all(self):
@@ -393,19 +474,19 @@ class TestDocValidator(unittest.TestCase):
     def test_get_docval_missing_arg(self):
         """Test that get_docval throws error if the matching docval argument is not found
         """
-        with self.assertRaisesRegex(ValueError, r"Function basic_add2 does not have docval argument 'arg3'"):
+        with self.assertRaisesWith(ValueError, "Function basic_add2 does not have docval argument 'arg3'"):
             get_docval(self.test_obj.basic_add2, 'arg3')
 
     def test_get_docval_missing_args(self):
         """Test that get_docval throws error if the matching docval arguments is not found
         """
-        with self.assertRaisesRegex(ValueError, r"Function basic_add2 does not have docval argument 'arg3'"):
+        with self.assertRaisesWith(ValueError, "Function basic_add2 does not have docval argument 'arg3'"):
             get_docval(self.test_obj.basic_add2, 'arg3', 'arg4')
 
     def test_get_docval_missing_arg_of_many_ok(self):
         """Test that get_docval throws error if the matching docval arguments is not found
         """
-        with self.assertRaisesRegex(ValueError, r"Function basic_add2 does not have docval argument 'arg3'"):
+        with self.assertRaisesWith(ValueError, "Function basic_add2 does not have docval argument 'arg3'"):
             get_docval(self.test_obj.basic_add2, 'arg2', 'arg3')
 
     def test_get_docval_none(self):
@@ -417,11 +498,37 @@ class TestDocValidator(unittest.TestCase):
     def test_get_docval_none_arg(self):
         """Test that get_docval throws error if there is no docval and an argument name is passed
         """
-        with self.assertRaisesRegex(ValueError, r'Function __init__ has no docval arguments'):
+        with self.assertRaisesWith(ValueError, 'Function __init__ has no docval arguments'):
             get_docval(self.test_obj.__init__, 'arg3')
 
+    def test_bool_type(self):
+        @docval({'name': 'arg1', 'type': bool, 'doc': 'this is a bool'})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
 
-class TestDocValidatorChain(unittest.TestCase):
+        res = method(self, arg1=True)
+        self.assertEqual(res, True)
+        self.assertIsInstance(res, bool)
+
+        res = method(self, arg1=np.bool_(True))
+        self.assertEqual(res, np.bool_(True))
+        self.assertIsInstance(res, np.bool_)
+
+    def test_bool_string_type(self):
+        @docval({'name': 'arg1', 'type': 'bool', 'doc': 'this is a bool'})
+        def method(self, **kwargs):
+            return popargs('arg1', kwargs)
+
+        res = method(self, arg1=True)
+        self.assertEqual(res, True)
+        self.assertIsInstance(res, bool)
+
+        res = method(self, arg1=np.bool_(True))
+        self.assertEqual(res, np.bool_(True))
+        self.assertIsInstance(res, np.bool_)
+
+
+class TestDocValidatorChain(TestCase):
 
     def setUp(self):
         self.obj1 = MyChainClass('base', [[1, 2], [3, 4], [5, 6]], [[10, 20]])
@@ -434,8 +541,8 @@ class TestDocValidatorChain(unittest.TestCase):
 
     def test_type_arg_wrong_type(self):
         """Test that passing an object for an argument that does not match a specific type raises an error"""
-        err_msg = r"incorrect type for 'arg1' \(got 'object', expected 'str or MyChainClass'\)"
-        with self.assertRaisesRegex(TypeError, err_msg):
+        err_msg = "MyChainClass.__init__: incorrect type for 'arg1' (got 'object', expected 'str or MyChainClass')"
+        with self.assertRaisesWith(TypeError, err_msg):
             MyChainClass(object(), [[10, 20], [30, 40], [50, 60]], [[10, 20]])
 
     def test_shape_valid_unpack(self):
@@ -452,8 +559,8 @@ class TestDocValidatorChain(unittest.TestCase):
         # shape after an object is initialized
         obj2.arg3 = [10, 20, 30]
 
-        err_msg = r"incorrect shape for 'arg3' \(got '\(3,\)', expected '\(None, 2\)'\)"
-        with self.assertRaisesRegex(ValueError, err_msg):
+        err_msg = "MyChainClass.__init__: incorrect shape for 'arg3' (got '(3,)', expected '(None, 2)')"
+        with self.assertRaisesWith(ValueError, err_msg):
             MyChainClass(self.obj1, obj2, [[100, 200]])
 
     def test_shape_none_unpack(self):
@@ -469,8 +576,8 @@ class TestDocValidatorChain(unittest.TestCase):
         obj2 = MyChainClass(self.obj1, [[10, 20], [30, 40], [50, 60]], [[10, 20]])
         obj2.arg3 = object()
 
-        err_msg = r"cannot check shape of object '<object object at .*>' for argument 'arg3' " \
-                  r"\(expected shape '\(None, 2\)'\)"
+        err_msg = (r"cannot check shape of object '<object object at .*>' for argument 'arg3' "
+                   r"\(expected shape '\(None, 2\)'\)")
         with self.assertRaisesRegex(ValueError, err_msg):
             MyChainClass(self.obj1, obj2, [[100, 200]])
 
@@ -489,8 +596,8 @@ class TestDocValidatorChain(unittest.TestCase):
         # shape after an object is initialized
         obj2.arg4 = [10, 20, 30]
 
-        err_msg = r"incorrect shape for 'arg4' \(got '\(3,\)', expected '\(None, 2\)'\)"
-        with self.assertRaisesRegex(ValueError, err_msg):
+        err_msg = "MyChainClass.__init__: incorrect shape for 'arg4' (got '(3,)', expected '(None, 2)')"
+        with self.assertRaisesWith(ValueError, err_msg):
             MyChainClass(self.obj1, [[100, 200], [300, 400], [500, 600]], arg4=obj2)
 
     def test_shape_none_unpack_default(self):
@@ -511,11 +618,7 @@ class TestDocValidatorChain(unittest.TestCase):
         # shape after an object is initialized
         obj2.arg4 = object()
 
-        err_msg = r"cannot check shape of object '<object object at .*>' for argument 'arg4' " \
-                  r"\(expected shape '\(None, 2\)'\)"
+        err_msg = (r"cannot check shape of object '<object object at .*>' for argument 'arg4' "
+                   r"\(expected shape '\(None, 2\)'\)")
         with self.assertRaisesRegex(ValueError, err_msg):
             MyChainClass(self.obj1, [[100, 200], [300, 400], [500, 600]], arg4=obj2)
-
-
-if __name__ == '__main__':
-    unittest.main()
